@@ -31,6 +31,7 @@ from pathlib import Path
 from typing import TYPE_CHECKING, Any, Iterable
 
 from airflow import settings
+from airflow.configuration import conf
 from airflow.task.priority_strategy import (
     PriorityWeightStrategy,
     airflow_priority_weight_strategies,
@@ -40,11 +41,14 @@ from airflow.utils.file import find_path_from_directory
 from airflow.utils.module_loading import import_string, qualname
 
 if TYPE_CHECKING:
+    from airflow.lineage.hook import HookLineageReader
+
     try:
         import importlib_metadata as metadata
     except ImportError:
         from importlib import metadata  # type: ignore[no-redef]
     from types import ModuleType
+    from typing import Generator
 
     from airflow.hooks.base import BaseHook
     from airflow.listeners.listener import ListenerManager
@@ -73,6 +77,7 @@ operator_extra_links: list[Any] | None = None
 registered_operator_link_classes: dict[str, type] | None = None
 registered_ti_dep_classes: dict[str, type] | None = None
 timetable_classes: dict[str, type[Timetable]] | None = None
+hook_lineage_reader_classes: list[type[HookLineageReader]] | None = None
 priority_weight_strategy_classes: dict[str, type[PriorityWeightStrategy]] | None = None
 """
 Mapping of class names to class of OperatorLinks registered by plugins.
@@ -174,8 +179,12 @@ class AirflowPlugin:
     # A list of timetable classes that can be used for DAG scheduling.
     timetables: list[type[Timetable]] = []
 
+    # A list of listeners that can be used for tracking task and DAG states.
     listeners: list[ModuleType | object] = []
 
+    # A list of hook lineage reader classes that can be used for reading lineage information from a hook.
+    hook_lineage_readers: list[type[HookLineageReader]] = []
+
     # A list of priority weight strategy classes that can be used for calculating tasks weight priority.
     priority_weight_strategies: list[type[PriorityWeightStrategy]] = []
 
@@ -262,28 +271,38 @@ def load_plugins_from_plugin_directory():
     """Load and register Airflow Plugins from plugins directory."""
     global import_errors
     log.debug("Loading plugins from directory: %s", settings.PLUGINS_FOLDER)
+    files = find_path_from_directory(settings.PLUGINS_FOLDER, ".airflowignore")
+    plugin_search_locations: list[tuple[str, Generator[str, None, None]]] = [("", files)]
 
-    for file_path in find_path_from_directory(settings.PLUGINS_FOLDER, ".airflowignore"):
-        path = Path(file_path)
-        if not path.is_file() or path.suffix != ".py":
-            continue
-        mod_name = path.stem
+    if conf.getboolean("core", "LOAD_EXAMPLES"):
+        log.debug("Note: Loading plugins from examples as well: %s", settings.PLUGINS_FOLDER)
+        from airflow.example_dags import plugins
 
-        try:
-            loader = importlib.machinery.SourceFileLoader(mod_name, file_path)
-            spec = importlib.util.spec_from_loader(mod_name, loader)
-            mod = importlib.util.module_from_spec(spec)
-            sys.modules[spec.name] = mod
-            loader.exec_module(mod)
-            log.debug("Importing plugin module %s", file_path)
+        example_plugins_folder = next(iter(plugins.__path__))
+        example_files = find_path_from_directory(example_plugins_folder, ".airflowignore")
+        plugin_search_locations.append((plugins.__name__, example_files))
 
-            for mod_attr_value in (m for m in mod.__dict__.values() if is_valid_plugin(m)):
-                plugin_instance = mod_attr_value()
-                plugin_instance.source = PluginsDirectorySource(file_path)
-                register_plugin(plugin_instance)
-        except Exception as e:
-            log.exception("Failed to import plugin %s", file_path)
-            import_errors[file_path] = str(e)
+    for module_prefix, plugin_files in plugin_search_locations:
+        for file_path in plugin_files:
+            path = Path(file_path)
+            if not path.is_file() or path.suffix != ".py":
+                continue
+            mod_name = f"{module_prefix}.{path.stem}" if module_prefix else path.stem
+
+            try:
+                loader = importlib.machinery.SourceFileLoader(mod_name, file_path)
+                spec = importlib.util.spec_from_loader(mod_name, loader)
+                mod = importlib.util.module_from_spec(spec)
+                sys.modules[spec.name] = mod
+                loader.exec_module(mod)
+
+                for mod_attr_value in (m for m in mod.__dict__.values() if is_valid_plugin(m)):
+                    plugin_instance = mod_attr_value()
+                    plugin_instance.source = PluginsDirectorySource(file_path)
+                    register_plugin(plugin_instance)
+            except Exception as e:
+                log.exception("Failed to import plugin %s", file_path)
+                import_errors[file_path] = str(e)
 
 
 def load_providers_plugins():
@@ -471,6 +490,25 @@ def initialize_timetables_plugins():
     }
 
 
+def initialize_hook_lineage_readers_plugins():
+    """Collect hook lineage reader classes registered by plugins."""
+    global hook_lineage_reader_classes
+
+    if hook_lineage_reader_classes is not None:
+        return
+
+    ensure_plugins_loaded()
+
+    if plugins is None:
+        raise AirflowPluginException("Can't load plugins.")
+
+    log.debug("Initialize hook lineage readers plugins")
+
+    hook_lineage_reader_classes = []
+    for plugin in plugins:
+        hook_lineage_reader_classes.extend(plugin.hook_lineage_readers)
+
+
 def integrate_executor_plugins() -> None:
     """Integrate executor plugins to the context."""
     global plugins
