@@ -20,6 +20,7 @@
 /* global moment */
 
 import { AnsiUp } from "ansi_up";
+import { getMetaValue, highlightByKeywords } from "src/utils";
 import { defaultFormatWithTZ } from "src/datetime_utils";
 
 export enum LogLevel {
@@ -38,6 +39,23 @@ export const logLevelColorMapping = {
   [LogLevel.CRITICAL]: "red.400",
 };
 
+const errorKeywords = getMetaValue("color_log_error_keywords")
+  .split(",")
+  .filter((keyword) => keyword.length > 0)
+  .map((keyword) => keyword.toLowerCase());
+const warningKeywords = getMetaValue("color_log_warning_keywords")
+  .split(",")
+  .filter((keyword) => keyword.length > 0)
+  .map((keyword) => keyword.toLowerCase());
+
+// Detect log groups which can be collapsed
+// Either in Github like format '::group::<group name>' to '::endgroup::'
+// see https://docs.github.com/en/actions/using-workflows/workflow-commands-for-github-actions#grouping-log-lines
+// Or in ADO pipeline like format '##[group]<group name>' to '##[endgroup]'
+// see https://learn.microsoft.com/en-us/azure/devops/pipelines/scripts/logging-commands?view=azure-devops&tabs=powershell#formatting-commands
+export const logGroupStart = / INFO - (::|##\[])group(::|\])([^\n])*/g;
+export const logGroupEnd = / INFO - (::|##\[])endgroup(::|\])/g;
+
 export const parseLogs = (
   data: string | undefined,
   timezone: string | null,
@@ -64,14 +82,8 @@ export const parseLogs = (
   const ansiUp = new AnsiUp();
   ansiUp.url_allowlist = {};
 
-  const urlRegex = /((https?:\/\/|http:\/\/)(?:(?!&#x27;|&quot;)[^\s])+)/g;
-  // Detect log groups which can be collapsed
-  // Either in Github like format '::group::<group name>' to '::endgroup::'
-  // see https://docs.github.com/en/actions/using-workflows/workflow-commands-for-github-actions#grouping-log-lines
-  // Or in ADO pipeline like format '##[group]<group name>' to '##[endgroup]'
-  // see https://learn.microsoft.com/en-us/azure/devops/pipelines/scripts/logging-commands?view=azure-devops&tabs=powershell#formatting-commands
-  const logGroupStart = / INFO - (::|##\[])group(::|\])([^\n])*/g;
-  const logGroupEnd = / INFO - (::|##\[])endgroup(::|\])/g;
+  const urlRegex =
+    /http(s)?:\/\/[\w.-]+(\.?:[\w.-]+)*([/?#][\w\-._~:/?#[\]@!$&'()*+,;=.%]+)?/g;
   // Coloring (blue-60 as chakra style, is #0060df) and style such that log group appears like a link
   const logGroupStyle = "color:#0060df;cursor:pointer;font-weight:bold;";
 
@@ -112,6 +124,13 @@ export const parseLogs = (
         line.includes(fileSourceFilter)
       )
     ) {
+      parsedLine = highlightByKeywords(
+        parsedLine,
+        errorKeywords,
+        warningKeywords,
+        logGroupStart,
+        logGroupEnd
+      );
       // for lines with color convert to nice HTML
       const coloredLine = ansiUp.ansi_to_html(parsedLine);
 
@@ -119,7 +138,8 @@ export const parseLogs = (
       const lineWithHyperlinks = coloredLine
         .replace(
           urlRegex,
-          '<a href="$1" target="_blank" style="color: blue; text-decoration: underline;">$1</a>'
+          (url) =>
+            `<a href="${url}" target="_blank" rel="noopener noreferrer" style="color: blue; text-decoration: underline;">${url}</a>`
         )
         .replace(logGroupStart, (textLine) => {
           const unfoldIdSuffix = "_unfold";
