@@ -185,6 +185,41 @@ const toSentenceCase = (camelCase: string): string => {
   return "";
 };
 
+const highlightByKeywords = (
+  parsedLine: string,
+  errorKeywords: string[],
+  warningKeywords: string[],
+  logGroupStart: RegExp,
+  logGroupEnd: RegExp
+): string => {
+  // Don't color log marker lines that are already highlighted.
+  if (logGroupStart.test(parsedLine) || logGroupEnd.test(parsedLine)) {
+    return parsedLine;
+  }
+
+  const lowerParsedLine = parsedLine.toLowerCase();
+  const red = (line: string) => `\x1b[1m\x1b[31m${line}\x1b[39m\x1b[0m`;
+  const yellow = (line: string) => `\x1b[1m\x1b[33m${line}\x1b[39m\x1b[0m`;
+
+  const containsError = errorKeywords.some((keyword) =>
+    lowerParsedLine.includes(keyword)
+  );
+
+  if (containsError) {
+    return red(parsedLine);
+  }
+
+  const containsWarning = warningKeywords.some((keyword) =>
+    lowerParsedLine.includes(keyword)
+  );
+
+  if (containsWarning) {
+    return yellow(parsedLine);
+  }
+
+  return parsedLine;
+};
+
 export {
   hoverDelay,
   finalStatesMap,
@@ -197,4 +232,5 @@ export {
   getStatusBackgroundColor,
   useOffsetTop,
   toSentenceCase,
+  highlightByKeywords,
 };
